/* Continuous Time Bayesian Network Reasoning and Learning Engine
 * Copyright (C) 2009 The Regents of the University of California
 *
 * Authored by Yu Fan, William Lam, Joon Lee, Christian Shelton, and Jing Xu
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
#include <iostream>
#include "graphic.h"
#include "markov.h"
#include "params.h"

using namespace std;

int main(int argc, char **argv)
{

	//to initialize or use the params.h class
	InitParams(argc, argv);
	//to see if the file opens or not
	string File;
	if(argc < 2)
		{
			cout << " you need to include a ctbn file" << endl;
			return 0;
		}
	File = ParamStr("File", "fail");
	ifstream ls(File.c_str());
	if(!ls.is_open()) return 0;

	//building the Markov to use in graphic program
	Markov ctbn(ls);

	//several different file to be written
	//dot extention for graphviz and eps extention for post-script
	ofstream file("output2.eps");
	ofstream file2("output.dot");
	ofstream file3("forthetest.eps");
	ofstream file4("forthetest2.eps");

	//creating the Graphic class name test
	Graphic test;

	//setting the font for the trajectory graph
	test.SetFontTrajectory(16, 10);

	//drawing the CTBN graph with rank system intergrated
	string filename = "output.dot";
	test.DrawCTBNRank(ctbn,file2, filename, file);
	file.close();

	//to create the Trajectory::Index
	const CTBNDyn * B = dynamic_cast<const CTBNDyn *>(ctbn.GetDynamics());
	Context context = Context();
	for (int i = 0; i < B->NumofNodes(); ++i)
		context = context + B->Node(i)->Domain();
	string name;
	Trajectory tr;
	tr.SetBeginTime(0);
	tr.SetEndTime(3);
	ctbn.Sample(tr);
	ofstream os("unit_test_traj.graph");
	tr.Save(os);


	//to draw the Index into trajectory graph
	test.DrawTrajectory(tr, context ,file3);
	test.DrawTrajectory(tr,context ,file4);

	//end of the file
	cout << "done" << endl;
	file.close();
	file2.close();
	file3.close();
	file4.close();
	return 0;
}
