/* Continuous Time Bayesian Network Reasoning and Learning Engine
 * Copyright (C) 2009 The Regents of the University of California
 *
 * Authored by Yu Fan, William Lam, Joon Lee, Christian Shelton, and Jing Xu
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
#ifndef _DEFINES_H
#define _DEFINES_H

// std/stdexcept.h is being included by std/typeinfo.h. We really don't
// want it, since it does not compile
//#include <typeinfo>

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <math.h>
#include <limits.h>
#include <float.h>


typedef long long int longlong;

#if defined __linux__ || __CYGWIN__
#define TIME_STRUCT longlong
#define GET_TIME    GetLinuxTime()
TIME_STRUCT GET_TIME;

#elif defined _WIN32
#include <limits>
#define NOMINMAX
#define WIN32_LEAN_AND_MEAN 
#include <windows.h>

#ifdef UNION
#undef UNION
#endif
#ifdef INTERSECTION
#undef INTERSECTION
#endif
#ifdef DIFFERENCE
#undef DIFFERENCE
#endif 


#define TIME_STRUCT longlong
#define GET_TIME GetWindowsTime()
/*
#define TIME_STRUCT longlong
#define GET_TIME	GetTime()
*/
TIME_STRUCT GET_TIME;
#else
//   #define TIME_STRUCT hrtime_t
//   #define GET_TIME    gethrtime()
// for mac os x
#include <mach/mach_time.h>
#define GET_TIME    mach_absolute_time()
#endif



#if defined(_WIN32)
#ifndef finite
#define finite(x) (_finite (x))
#endif

#ifndef isfinite 
#define isfinite(x) (_finite (x))
#endif

#ifndef isnan
#define isnan(d) (_isnan(d))
#endif

#ifndef __func__
#define __func__ __FUNCTION__
#endif

#ifndef isinf 
#define isinf(x) (!_finite(x) && !_isnan(x)) 
#endif 

#ifndef INFINITY
#define INFINITY (std::numeric_limits<double>::infinity())
#endif

extern const unsigned long win32nan[2];
double nan(const char *);
float nanf(const char *);
long double nanl(const char *);
#endif



#endif // _DEFINES_H
