/* Continuous Time Bayesian Network Reasoning and Learning Engine
 * Copyright (C) 2009 The Regents of the University of California
 *
 * Authored by Yu Fan, William Lam, Joon Lee, Christian Shelton, and Jing Xu
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
#ifndef _SAMPLINGINF_H_
#define _SAMPLINGINF_H_

#include "inference.h"
#include "sampler.h"
#include "trajectory.h"
#include "context.h"
#include "markov.h"
#include "trajectory.h"
#include "ss.h"
#include <vector> 


class SamplingInf : public Inference{
public:
	SamplingInf();
	virtual ~SamplingInf();
	virtual SamplingInf *Clone() const;
	virtual void SetProcess(const Process *pr);
	virtual void SetTrajectory(const Trajectory *tr);
	virtual void SetSampler(const Sampler *s);
	void SetNumSample(int n);
	void SetTimeLimit(double t, bool isgibbs=false);

	virtual double Filter(const Instantiation &x, double t, 
				bool log=false);
	virtual double Smooth(const Instantiation &x, double t, 
				bool log=false);
	virtual double Prob(double t, bool log=false);
	virtual void AddExpSuffStats(SS *ss, double w=1.0);
	virtual void AddExpSuffStats(const Dynamics *dyn, SS *ss,
					double w=1.0);
	virtual void AddExpSuffStats(const RV *p0, SS *ss, 
					double w=1.0);

	//double LLH(const std::vector<Trajectory> &tr, 
	//		const std::vector<double> &w) const;
	double CalcQuery(QueryCalculator &calc);
protected:
	void Sample();
	void CleanSamples();
	const Markov *p;
	Sampler *sampler; //SamplingInf own this pointer

	int numofsamples;
	double timelimit;
	bool isgibbs;
	std::vector<Trajectory> traj;
	std::vector<double> weights;
};

#endif

