/* Continuous Time Bayesian Network Reasoning and Learning Engine
 * Copyright (C) 2009 The Regents of the University of California
 *
 * Authored by Yu Fan, William Lam, Joon Lee, Christian Shelton, and Jing Xu
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
#ifndef _STRUCTURE_H_
#define _STRUCTURE_H_

#include <vector>
#include "ctbndyn.h"
#include "context.h"
//the structure class copy node2var, var2node and children
//from class ctbndyn. It can be used when the structure can
//not be accessed directly.
class Structure {
friend class CTBNDyn;
friend class BN;
friend class StructureSearch;
friend class GraphEditSearch;
public: 
	Structure();
	virtual ~Structure();
	const std::vector<int>& GetParent(int child) const;
	const std::vector<int>& GetChildren(int parent) const;
	int Node2Var(int index) const;
	int Var2Node(int index) const;
	const std::vector<std::vector<int> >& GetAdjLists() const;
	int HammingDist(const Structure &other);
	std::vector<int> GetVarList() const;
	int GetNumParams(const Context& c) const;
  	bool IsCyclic() const;
	void Print(std::ostream& os) const;
protected:
	std::vector<int> node2var; //mapping from index into "nodes" to the variable id represented
	std::vector<std::vector<int> > parentlist; //parents of nodes
	//cache the parents variable id of each node index
	std::map<int, int> var2node; //mapping from vraiable id to the index into "nodes"
	std::map<int,std::vector<int> > children; // cache of
	// the indexes into "nodes" 
	// of the children for a particular variable id
	// (Note this one is slightly odd as it maps a variable id
	// into a list of node-indexes (not back into varids)
	std::vector<int> empty;
};
#endif
