/* Continuous Time Bayesian Network Reasoning and Learning Engine
 * Copyright (C) 2009 The Regents of the University of California
 *
 * Authored by Yu Fan, William Lam, Joon Lee, Christian Shelton, and Jing Xu
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
#include "rvsimple.h"

using namespace std;

RVSimple::RVSimple() : StreamObj() {
}

RVSimple::RVSimple(istream &is) : StreamObj() {
}

RVSimple::~RVSimple() {
}

void RVSimple::Load(istream &is) {
}

void RVSimple::Save(ostream &os) const {
}

//------

RVCondSimple::RVCondSimple() : StreamObj() {
}

RVCondSimple::RVCondSimple(std::istream &is) : StreamObj() {
}

RVCondSimple::~RVCondSimple() {
}

void RVCondSimple::Load(istream &is) {
}

void RVCondSimple::Save(ostream &os) const {
}

//------

SOBJCLASSDEF(RVCSCompSS)

RVCSCompSS::RVCSCompSS(int nc) : ss(nc) {
	for(int i=0;i<nc;i++) ss[i] = NULL;
}

RVCSCompSS::RVCSCompSS(const RVCSCompSS &x) : ss(x.ss.size()) {
	int nc = x.ss.size();
	for(int i=0;i<nc;i++) if (x.ss[i] != NULL) ss[i] = x.ss[i]->Clone();
}

RVCSCompSS::RVCSCompSS(istream &is) {
	int nc;
	is >> nc;
	for(int i=0;i<nc;i++) {
		SS *sptr = dynamic_cast<SS *>(StreamObj::LoadPtr(is));
		ss.push_back(sptr);
	}
}

RVCSCompSS::~RVCSCompSS() {
	int nc = ss.size(); 
	for(int i=0;i<nc;i++)
		if (ss[i]!=NULL) delete ss[i];
}

void RVCSCompSS::Load(istream &is) {
	int nc = ss.size(); 
	for(int i=0;i<nc;i++)
		if (ss[i]!=NULL) delete ss[i];
	is >> nc;
	ss.resize(nc);
	for(int i=0;i<nc;i++)
		ss[i] = dynamic_cast<SS *>(StreamObj::LoadPtr(is));
}

void RVCSCompSS::Save(ostream &os) const {
	int nc = ss.size();
	os << nc;
	for(int i=0;i<nc;i++) {
		os << os.fill();
		ss[i]->SavePtr(os);
	}
}

RVCSCompSS *RVCSCompSS::Clone() const {
	return new RVCSCompSS(*this);
}

RVCSCompSS &RVCSCompSS::operator=(const RVCSCompSS &x) {
	if (&x!=this) {
		int nc = ss.size();
		for(int i=0;i<nc;i++)
			if (ss[i]!=NULL) delete ss[i];
		nc = x.ss.size();
		ss.resize(nc);
		for(int i=0;i<nc;i++)
			if (x.ss[i]==NULL) ss[i] = NULL;
			else ss[i] = x.ss[i]->Clone();
	}
	return *this;
}

void RVCSCompSS::Scale(double w) {
	int n = ss.size();
	for(int i=0; i<n; i++)
		ss[i]->Scale(w);
}

void RVCSCompSS::AddSS(const SS* nss, double w) {
	const RVCSCompSS* rss = dynamic_cast<const RVCSCompSS*>(nss);
	int n = ss.size();
	for(int i=0; i<n; i++)
		ss[i]->AddSS(rss->ss[i], w);
}
