/* Continuous Time Bayesian Network Reasoning and Learning Engine
 * Copyright (C) 2009 The Regents of the University of California
 *
 * Authored by Yu Fan, William Lam, Joon Lee, Christian Shelton, and Jing Xu
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
#include "streamextra.h"
#include <string.h>

// the (few) definitions for that in streamextra.h

std::map<std::string, StreamObj::CreatorFn> &StreamObj::loadtable() {
	static std::map<std::string, StreamObj::CreatorFn> *_loadtable
		= new std::map<std::string, StreamObj::CreatorFn>();
	return *_loadtable;
}

StreamObj::StreamObj() {
}

StreamObj::~StreamObj() {
}

const char *StreamObj::myID() const {
	return "StreamObj";
}

const char *StreamObj::IDname
		= StreamObj::addID("StreamObj",StreamObj::Create);

void StreamObj::Save(std::ostream &os) const {
}

void StreamObj::SavePtr(std::ostream &os) const {
	os << myID() << os.fill();
	Save(os);
}

StreamObj *StreamObj::LoadPtr(std::istream &is) {
	std::string name;
	is >> name;
        //std::cout << "name : " << name << std::endl;
	std::map<std::string, CreatorFn>::iterator rec
			= loadtable().find(name);
	if (rec == loadtable().end()) return NULL;
	else return rec->second(is);
        
}

StreamObj *StreamObj::Create(std::istream &is) {
	return new StreamObj();
}

const char *StreamObj::addID(const char *classname, CreatorFn fn) {
	loadtable()[std::string(classname)] = fn;
	//std::cout << "LOADTABLE adding " << classname << std::endl;
	return classname;
}
const char *StreamObj::addID(const char *classname, const char *classname2,
					CreatorFn fn) {
	char *newname = new char[strlen(classname)+strlen(classname2)+2];
	strcpy(newname,classname);
	strcat(newname,".");
	strcat(newname,classname2);
	//std::cout << "LOADTABLE adding " << newname << std::endl;
	loadtable()[std::string(newname)] = fn;
	return newname;
}

char *StreamObj::killSpaces(const char *classname) {
	char *ret = new char[strlen(classname)+1];
	strcpy(ret,classname);
	for(int i=0;ret[i];i++)
		if (ret[i]==' ') ret[i]='_';
	return ret;
}

namespace {
	char infstring[] = "inf";
	char nanstring[] = "NaN";

	// the "C" means classic, and it should really be based
	// on the current locale, but currently the "C" is ignored anyway!
	nannum_put<char,infstring,nanstring> *putfacet = 
			new nannum_put<char,infstring,nanstring>("C");
	nannum_get<char,infstring,nanstring> *getfacet = 
			new nannum_get<char,infstring,nanstring>("C");
}

void addnan(std::basic_ios<char> &str) {
	std::locale withread(str.getloc(),putfacet);
	std::locale withboth(withread,getfacet);
	str.imbue(withboth);
}

