/* Continuous Time Bayesian Network Reasoning and Learning Engine
 * Copyright (C) 2009 The Regents of the University of California
 *
 * Authored by Yu Fan, William Lam, Joon Lee, Christian Shelton, and Jing Xu
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
//This code learns the parameters of a CTBN model
//given fully observed data.

#include "markov.h"
#include "trajectory.h"
#include "params.h"
#include <fstream>

using namespace std;

int main(int argc, char**argv) {
	InitParams(argc, argv);

	string ctbnfile = ParamStr("ctbn", "");
	if(ctbnfile == "") {
		cout << "CTBN model is missing" << endl;
		exit(1);
	}
	int n = ParamInt("numofsample", 1000);
	double begintime = ParamDouble("begintime" , 0.0);
	double endtime = ParamDouble("endtime", 5.0);
	ifstream fin(ctbnfile.c_str());
	if(!fin.good()) {
		cout << "bad CTBN file" << endl;
		exit(1);
	}
	//load CTBN model from the file
	Markov ctbn(fin);

	//generate data
	vector<Trajectory> data;
	for(int i=0; i<n; i++) {
		Trajectory tr;
		tr.SetBeginTime(begintime);
		tr.SetEndTime(endtime);
		ctbn.Sample(tr);
		data.push_back(tr);
	}

	//Calculate sufficient statistics of the data
	vector<double> w(n, 1.0);
	SS *ss = ctbn.SuffStats(data, w);
	//random choose parameters
	ctbn.Scramble();
	//learn parameters from complete data
	ctbn.Maximize(ss);
	ctbn.Save(cout);
	if(ss!=NULL) delete ss;

	return 0;
}
